! Time-stamp: <fe1.fsl.noaa.gov:/home/jacobson/TM5/proj/ctemis/trunk/src/emission_co2_fires.F90: 21 Sep 2011 (Wed) 01:15:05 GMT>

! todo:
!   Find out how to determine HDF variable name on the fly (is BB or BB_co2)
!     - note that files historically have a mixture of these names, even within a single run
!     - this tends to generate erroneous (and confusing) error messages

#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
#include "tm5.inc"

! might need this for sfn2index   #include "hdf.f90"

! Compute wildfire emissions of CO2 in mol m-2 s-1.  Positive is source to atmosphere.

module emission_co2_fires

  use GO,		only: gol, goErr, goPr, goLabel
  use Dims,		only: nregions, nlon360, nlat180
  use emission_common,	only: enkf_parms, field_1x1, EmisInputDir, co2_fires
  use emission_common,	only: nmembersloc

  implicit none

  character(len=*), parameter	:: mname = 'emission_co2_fires'

  private
  character(len=2048)		:: fires_prefix
  character(len=256)		:: fires_varname

  public	:: emission_co2_fires_init, emission_co2_fires_done, emission_co2_fires_calc

  ! use this to potentially override the EmisInputDir
  character(len=1024)	:: FiresInputDir

contains

  subroutine emission_co2_fires_init(status)

    use GO,		only: TrcFile, Init, Done, ReadRc
    use global_data,	only: rcfile
    use dims,		only: im,jm,nregions

    implicit none

    character(len=*), parameter	:: rname = mname//'/emission_co2_fires_init'

    integer			:: region, imr, jmr, n
    integer, intent(out)	:: status
    type(TrcFile)		:: rcF

    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'fires.input.dir', FiresInputDir ,status ,default=EmisInputDir)
    IF_ERROR_RETURN(status=1)
    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

    write (gol,'("[emission_co2_fires_init]	fires input files to be read from """,a,""".")') trim(FiresInputDir)
    call goPr

  end subroutine emission_co2_fires_init

  subroutine emission_co2_fires_calc(parms,co2_fires,flux_updated,status)

    use dims,			only : sec_month, idate, newmonth, sec_year, newsrun
    use MDF,			only : MDF_HDF4
    use ReadFromFile_MDF,	only : ReadFromFile
    use emission_data,		only : chardate, flux_to_gridbox_per_month
    use go_string,		only : goNum2str
    use chem_param,		only : xmc

    implicit none

    character(len=*), parameter			:: rname = mname//'/emission_co2_fires_calc'

    ! parms actually not used here since we do not optimize FIRES emissions
    type(enkf_parms), dimension(:), intent(in)	:: parms

    type(field_1x1), dimension(:), intent(out)	:: co2_fires
    type(field_1x1)				:: temp_read
    integer, intent(out)			:: status
    logical, intent(out)			:: flux_updated

    character(len=10)				:: cdate
    integer					:: imember
    real					:: month2year


    flux_updated = .false.

    if(newmonth.or.newsrun) then

       flux_updated = .true.

       cdate=trim(goNum2str(idate(1),'(i4.4)'))//trim(goNum2str(idate(2),'(i2.2)'))


       ! TODO:  were SDS_DATE attributes required in readfromfile call?
       !       call ReadFromFile(fname=trim(FiresInputDir)//'/BBguido1x1.hdf'//trim(cdate),ftype=MDF_HDF4,vname='BB',buffer=temp_read%surf,status=status,sds_date=(/idate(1),idate(2),1,0,0,0/))

       ! units of biomass burning are gC m-2 s-1
       call ReadFromFile_vname_uncertain(fname=trim(FiresInputDir)//'/BBguido1x1.hdf'//trim(cdate),ftype=MDF_HDF4,vname1='BB',vname2='BB_co2',buffer=temp_read%surf,status=status)
       IF_NOTOK_RETURN(status=1)

       month2year = sec_year/sec_month

       do imember=1,nmembersloc
          co2_fires(imember)%surf=temp_read%surf/1.0e3 ! from gC m-2 s-1 to kgC m-2 s-1

          call flux_to_gridbox_per_month(co2_fires(imember)%surf) ! convert from kgC m-2 s-1 to kgC 1x1_gridbox-1 month-1

          write (gol,'("[emission_co2_fires_calc]	member ",i4,": wildfire emission global integral ",f12.4," PgC/yr.")') imember,sum(co2_fires(imember)%surf)*1.0e-12*month2year
          call goPr

       end do

    endif ! newmonth or newsrun

  end subroutine emission_co2_fires_calc

  subroutine emission_co2_fires_done(status)

    use dims,		only: nregions

    implicit none

    integer, intent(out)			:: status

    character(len=*), parameter	:: rname = mname//'/emission_co2_fires_done'

    status=0
    return

  end subroutine emission_co2_fires_done

  subroutine ReadFromFile_vname_uncertain(fname,ftype,vname1,vname2,buffer,status)

    use GO,			only : gol, goErr, goPr, goLabel
    use MDF

    implicit none

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname1  ! variable name
    character(len=*), intent(in)	:: vname2  ! variable name
    real, intent(out)			:: buffer(:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_vname_uncertain'


    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call my_MDF_Inq_VarID(hid,vname1,vid,status,.TRUE.)
    IF_ERROR_RETURN(status=1)     ! status < 0 means variable not found; > 0 is a real error
    if(status .lt. 0) then
       status=0
       call MDF_Inq_VarID(hid,vname2,vid,status)
       IF_NOTOK_RETURN(status=1)
    end if

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status=0

  end subroutine ReadFromFile_vname_uncertain

  subroutine my_MDF_Inq_VarID( hid, name, varid, status, fail_gracefully )

    use MDF

    ! --- in/out -------------------------------------

    integer, intent(in)                           ::  hid
    character(len=*), intent(in)                  ::  name
    integer, intent(out)                          ::  varid
    integer, intent(out)                          ::  status
    logical, intent(in), optional                 ::  fail_gracefully

    ! --- const --------------------------------------

    character(len=*), parameter  ::  rname = mname//'/MDF_Inq_VarID'
    
    ! --- local --------------------------------------

    integer                         ::  nvar, ivar
    character(len=64)               ::  varname

    ! --- begin --------------------------------------
    
    ! dummy ...
    varid = -1

    ! number of variables:
    call MDF_Inquire( hid, status, nVariables=nvar )
    IF_NOTOK_RETURN(status=1)
    ! list variables ?
    if ( nvar > 0 ) then
      ! loop over variables:
      do ivar = 1, nvar
        ! get name:
        call MDF_Inquire_Variable( hid, ivar, status, name=varname )
        IF_NOTOK_RETURN(status=1)
        ! similar ?
        if ( trim(name) == trim(varname) ) then
          ! store current id:
          varid = ivar
          ! leave:
          exit
        end if
      end do  ! variables
    end if
    
    ! ok
    status = 0

    ! check ...
    if ( varid < 0 ) then
       status=varid
       if (.not. present(fail_gracefully)) then
          write (gol,'("no variable `",a,"` found in file.")') trim(name); call goErr
          TRACEBACK; status=varid; return
       end if
    end if
    
  end subroutine my_MDF_Inq_VarID
  

end module emission_co2_fires

