! TODO
!  The emissions averaging code here needs to be moved to user_output_flux1x1
!  Keep parameters, other "real" emission_common stuff.

!### macros #####################################################
#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
                                                                                                                                                     
#include "tm5.inc"
!#################################################################

! TODO:  this branch module exists only to change the value of nmembersloc,
! which should be set programmatically anyway.

module emission_common

  use GO,				only : gol, goErr, goPr, goLabel
  use Dims,            	      		only : nlon360,nlat180


  ! This structure is used to hold emission module results and also
  ! to hold mean fluxes.

  type field_1x1
     real*8, dimension(nlon360,nlat180)			:: surf
     integer						:: n
     real*8						:: wda
  end type field_1x1


  ! This particular parameter structure is appropriate for a map of scaling factors.

  type enkf_parms
     real, dimension(nlon360,nlat180)			:: surf
  end type enkf_parms

  character(len=1024)					:: EmisInputDir
  character(len=2048)					:: FluxOutFile = ''
  integer						:: nmembersloc=5 ! for ensemble only (TODO: remove hard-coding)


  ! Each member of co2_bio contains a (scaled) flux field.  Integer n unused.

  type(field_1x1), dimension(:), allocatable		:: co2_bio
  type(field_1x1), dimension(:), allocatable		:: co2_ocn
  type(field_1x1), dimension(:), allocatable		:: co2_ff
  type(field_1x1), dimension(:), allocatable		:: co2_fires


  ! Each member of enkf_parms contains the set of parameters for a given ensemble member.

  type(enkf_parms),dimension(:), allocatable		:: parms

  ! 1x1 degree global map of regions.  Maps scaling-factor parameters to geographic location.
  ! not using field_1x1 structure since values are integers
  integer,dimension(:,:), allocatable			:: region_map

  ! co2_bio_m is the flue mean accumulator, with n being the number of samples.

  type(field_1x1)					:: co2_bio_m
  type(field_1x1)					:: co2_ocn_m
  type(field_1x1)					:: co2_ff_m
  type(field_1x1)					:: co2_fires_m

  integer, dimension(6)					:: idate_api=-1 ! averaging period start
  integer, dimension(6)					:: idate_ape=-1 ! averaging period end
  real							:: flux1x1_dhour ! averaging period length (hours)
  logical                                               :: flux1x1_data = .false.     ! signal for flux1x1 output

  private

  public						:: idate_api,idate_ape
  public						:: field_1x1, enkf_parms
  public						:: co2_bio, co2_bio_m
  public						:: co2_ocn, co2_ocn_m
  public						:: co2_ff, co2_ff_m
  public						:: co2_fires, co2_fires_m
  public						:: flux1x1_dhour
  public						:: flux1x1_data
  public						:: parms
  public						:: region_map

  public						:: emission_averagers_reset, emission_averagers_calc, emission_averagers_save
  public						:: EmisInputDir
  public						:: nmembersloc

  
  character(len=*), parameter				:: mname = 'emission_common'
  private						:: mname

contains

  subroutine emission_averagers_calc

    character(len=*), parameter	:: rname = mname//'/emission_averagers_calc'
    integer 			:: j

    ! compute average; note that units are already mol m-2 s-1 

    if(co2_bio_m%n .gt. 0) then
       co2_bio_m%surf = co2_bio_m%surf / co2_bio_m%wda
       co2_bio_m%n = 0
       co2_bio_m%wda = 0.
    end if

    if(co2_ocn_m%n .gt. 0) then
       co2_ocn_m%surf = co2_ocn_m%surf / co2_ocn_m%wda
       co2_ocn_m%n = 0
       co2_ocn_m%wda = 0.
    end if

    if(co2_fires_m%n .gt. 0) then
       co2_fires_m%surf = co2_fires_m%surf / co2_fires_m%wda
       co2_fires_m%n = 0
       co2_fires_m%wda = 0.
    end if

    if(co2_ff_m%n .gt. 0) then
       co2_ff_m%surf = co2_ff_m%surf / co2_ff_m%wda
       co2_ff_m%n = 0
       co2_ff_m%wda = 0.
    end if

  end subroutine emission_averagers_calc

  subroutine emission_averagers_save(status)

    use ParTools, 	only : myid, root
    use MDF, 		only : MDF_Open
    use MDF, 		only : MDF_Create, MDF_Close, MDF_EndDef
    use MDF, 		only : MDF_NETCDF, MDF_REPLACE, MDF_GLOBAL, MDF_CHAR, MDF_INT, MDF_FLOAT, MDF_UNLIMITED, MDF_READ, MDF_DOUBLE
    use MDF, 		only : MDF_Put_Att
    use MDF, 		only : MDF_Def_Dim
    use MDF, 		only : MDF_Inq_Dimid, MDF_Inquire_Dimension
    use MDF, 		only : MDF_Inq_VarID
    use MDF, 		only : MDF_Def_Var, MDF_Put_Var, MDF_Get_Var
    use global_data, 	only : outdir
    use dims, 		only : itau
    use datetime, 	only : tau2date, date2tau, idate2ddate

    integer,intent(out)			:: status

    character(len=200)			:: dsname,attname,comp,pp

    character(len=*), parameter		:: rname = mname//'/emission_averagers_save'

    integer				:: i
    integer				:: hid
    integer				:: dim_lon_id, dim_lat_id, dim_date_id, dim_date_comps
    integer				:: var_flux_ff, var_flux_bio, var_flux_ocn, var_flux_fires, var_lon, var_lat, var_date, var_comps, var_decdate
    integer, dimension(6)		:: idate_epoch, idate_middle
    integer				:: itau_epoch, itau_api, itau_ape
    real				:: itau_middle ! real since it is an average
    real				:: date_value ! days-since value corresponding to itau_middle, to be written
    real*8				:: decimal_date
    character(len=256)			:: progstring
    character(len=1024)			:: attstring
    real,dimension(2)			:: axis_range
    real*8,parameter			:: fillval_r8 = -1.e34
    integer, dimension(8)		:: isysdate
    character(len=256)			:: sysdate 

    if(myid .ne. root) return

    call emission_averagers_calc ! OK if called repeatedly

    call tau2date(itau,idate_ape) ! set averaging period end to right now
    itau_ape=itau
    call date2tau(idate_api,itau_api)

    idate_epoch = (/2000,1,1,0,0,0/)
    call date2tau(idate_epoch,itau_epoch)

    itau_middle = 0.5*(itau_api + itau_ape) - itau_epoch
    date_value = itau_middle/86400.

    itau_middle = 0.5*(itau_api + itau_ape) ! note recycling of itau_middle here
    call tau2date(int(itau_middle),idate_middle)

    decimal_date=idate2ddate(idate_middle)

    write (FluxOutFile,'(a,"/flux1x1_",i4.4,4i2.2,"_",i4.4,4i2.2,".nc")') &
                trim(outdir), idate_api(1:5), idate_ape(1:5)

    call MDF_Create( trim(FluxOutFile), MDF_NETCDF, MDF_REPLACE, hid, status )
    IF_NOTOK_RETURN(status=1)

    ! Global Attributes 

    write(attstring,'(i4.4,"/",i2.2,"/",i2.2," ",i2.2,":",i2.2,":",i2.2, " UTC")') idate_api
    call MDF_Put_Att(hid, MDF_GLOBAL,"averaging_period_start_date",values=trim(attstring),status=status)
    IF_NOTOK_RETURN(status=1)

    write(attstring,'(i4.4,"/",i2.2,"/",i2.2," ",i2.2,":",i2.2,":",i2.2, " UTC")') idate_ape
    call MDF_Put_Att(hid, MDF_GLOBAL,"averaging_period_end_date",values=trim(attstring),status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Put_Att(hid, MDF_GLOBAL,"averaging_period_length_hours",values=flux1x1_dhour,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Put_Att(hid, MDF_GLOBAL,"Notes",values="This file contains CarbonTracker surface CO2 fluxes averaged over each time interval.  The times on the date axis are the centers of each averaging period.",status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Put_Att(hid, MDF_GLOBAL,"Disclaimer",values="CarbonTracker is an open product of the NOAA Earth System Research Laboratory using data from the Global Monitoring Division greenhouse gas observational network and collaborating institutions.  Model results including figures and tabular material found on the CarbonTracker website may be used for non-commercial purposes without restriction, but we request that the following acknowledgement text be included in documents or publications made using CarbonTracker results: ""CarbonTracker results provided by NOAA/ESRL, Boulder, Colorado, USA, http://carbontracker.noaa.gov.""  Since we expect to continuously update the CarbonTracker product, it is important to identify which version you are using.  To provide accurate citation, please include the version of the CarbonTracker release in any use of these results.  The CarbonTracker team welcomes special requests for data products not offered by default on this website, and encourages proposals for collaborative activities.  Contact us at carbontracker.team@noaa.gov.",status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Put_Att(hid, MDF_GLOBAL,"email",values="carbontracker.team@noaa.gov",status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Put_Att(hid, MDF_GLOBAL,"url",values="http://carbontracker.noaa.gov",status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Put_Att(hid, MDF_GLOBAL,"institution",values="NOAA Earth System Research Laboratory",status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Put_Att(hid, MDF_GLOBAL,"Conventions",values="CF-1.5",status=status)
    IF_NOTOK_RETURN(status=1)

    call getarg (0, progstring, status)
    call date_and_time(values = isysdate)
    write (sysdate, '(i4.4,"-",i2.2,"-",i2.2," ",i2.2,":",i2.2,":",i2.2," UTC")') &
           isysdate(1), isysdate(2), isysdate(3), isysdate(5), isysdate(6), isysdate(7)
    write(attstring,'("Created ",a," by ",a,".")') trim(sysdate),trim(progstring)
    call MDF_Put_Att(hid, MDF_GLOBAL,"history",values=attstring,status=status)
    IF_NOTOK_RETURN(status=1)

    ! define dimensions:                                                                                                                                                                                            
    call MDF_Def_Dim( hid, 'lon', 360 , dim_lon_id, status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Def_Dim( hid, 'lat', 180 , dim_lat_id, status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Def_Dim( hid, 'date', MDF_UNLIMITED , dim_date_id, status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Def_Dim( hid, 'date_components', 6 , dim_date_comps, status )
    IF_NOTOK_RETURN(status=1)

    ! dimension variables:                                                                                                                                                                                                    
    call MDF_Def_Var( hid, 'lon', MDF_DOUBLE, (/dim_lon_id/), var_lon, status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_lon,"units",values="degrees_east",status=status)
    IF_NOTOK_RETURN(status=1)
    axis_range = (/ -180., 180./)
    call MDF_Put_Att(hid, var_lon,"actual_range",values=axis_range,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Def_Var( hid, 'lat', MDF_DOUBLE, (/dim_lat_id/), var_lat, status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_lat,"units",values="degrees_north",status=status)
    IF_NOTOK_RETURN(status=1)
    axis_range = (/ -90., 90./)
    call MDF_Put_Att(hid, var_lat,"actual_range",values=axis_range,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Def_Var( hid, 'date', MDF_DOUBLE, (/dim_date_id/), var_date, status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_date,"units",values="days since 2000-01-01 00:00:00 UTC",status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Def_Var( hid, 'decimal_date', MDF_DOUBLE, (/dim_date_id/), var_decdate, status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_decdate,"units",values="years",status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Def_Var( hid, 'date_components', MDF_INT, (/dim_date_comps, dim_date_id/), var_comps, status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_comps,"units",values="NA",status=status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_comps,"comment",values="Calendar date components as integers.  Times and dates are UTC.",status=status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_comps,"order",values="year, month, day, hour, minute, second",status=status)
    IF_NOTOK_RETURN(status=1)

    ! define variables:                                                                                                                                                                                                    
    comp='ff'
    dsname='fossil_flux_imp'
    
    call MDF_Def_Var( hid, trim(dsname), MDF_DOUBLE, (/dim_lon_id,dim_lat_id,dim_date_id/), var_flux_ff, deflate_level=9, status=status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_flux_ff,"units",values="mol m-2 s-1",status=status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_flux_ff,"_FillValue",values=fillval_r8,status=status)
    IF_NOTOK_RETURN(status=1)

    comp='bio'
    dsname='bio_flux_opt'

    call MDF_Def_Var( hid, trim(dsname), MDF_DOUBLE, (/dim_lon_id,dim_lat_id,dim_date_id/), var_flux_bio, deflate_level=9, status=status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_flux_bio,"units",values="mol m-2 s-1",status=status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_flux_bio,"_FillValue",values=fillval_r8,status=status)
    IF_NOTOK_RETURN(status=1)

    comp='fires'
    dsname='fire_flux_imp'

    call MDF_Def_Var( hid, trim(dsname), MDF_DOUBLE, (/dim_lon_id,dim_lat_id,dim_date_id/), var_flux_fires, deflate_level=9, status=status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_flux_fires,"units",values="mol m-2 s-1",status=status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_flux_fires,"_FillValue",values=fillval_r8,status=status)
    IF_NOTOK_RETURN(status=1)

    comp='ocn'
    dsname='ocn_flux_opt'

    call MDF_Def_Var( hid, trim(dsname), MDF_DOUBLE, (/dim_lon_id,dim_lat_id,dim_date_id/), var_flux_ocn, deflate_level=9, status=status )
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_flux_ocn,"units",values="mol m-2 s-1",status=status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Att(hid, var_flux_ocn,"_FillValue",values=fillval_r8,status=status)
    IF_NOTOK_RETURN(status=1)

    ! finished definition:                                                                                                                                                                                          
    call MDF_EndDef( hid, status )
    IF_NOTOK_RETURN(status=1)

    call MDF_Put_Var( hid, var_lon, (/(i+0.5,i=-180,179)/), status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Var( hid, var_lat, (/(i+0.5,i=-90,89)/), status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Var( hid, var_date, (/(date_value)/), status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Var( hid, var_decdate, (/(decimal_date)/), status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Put_Var( hid, var_comps, (/(idate_middle)/), status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Put_Var( hid, var_flux_ff, co2_ff_m%surf, status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Var( hid, var_flux_bio, co2_bio_m%surf, status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Var( hid, var_flux_ocn, co2_ocn_m%surf, status)
    IF_NOTOK_RETURN(status=1)
    call MDF_Put_Var( hid, var_flux_fires, co2_fires_m%surf, status)
    IF_NOTOK_RETURN(status=1)

    ! close file:                                                                                                                                                                                                   
    call MDF_Close( hid, status )
    IF_NOTOK_RETURN(status=1)

    call tau2date(itau,idate_api) ! for next averaging period

    call emission_averagers_reset

    write (gol,'("[emission_averagers_save]  Done writing arrays and closing output file.")'); call goPr

    status=0
    
  end subroutine emission_averagers_save

  subroutine emission_averagers_reset

    co2_bio_m%surf = 0.
    co2_bio_m%n = 0
    co2_bio_m%wda = 0.
    
    co2_ocn_m%surf = 0.
    co2_ocn_m%n = 0
    co2_ocn_m%wda = 0.
    
    co2_fires_m%surf = 0.
    co2_fires_m%n = 0
    co2_fires_m%wda = 0.
       
    co2_ff_m%surf = 0.
    co2_ff_m%n = 0
    co2_ff_m%wda = 0.
       
  end subroutine emission_averagers_reset

end module emission_common
